//  Copyright (C) 2005 SoftKAM. All Rights Reserved.
//
//  DeskZoom is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
//  USA.

#pragma comment(linker,"/FILEALIGN:0x200")

#include <windows.h>
#include "resource.h"

#define ZOOM_TIMER_ID			10

// Global Variables
HINSTANCE	g_hInst;
HWND		g_hWnd;
BOOL		g_bNoRepaint;
int			g_nFreq;
int			g_nZoomLevel;
// Foward declarations of functions included in this code module:
LRESULT CALLBACK	WndProc(HWND, UINT, WPARAM, LPARAM);
INT_PTR CALLBACK	CustomizeProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);
INT_PTR CALLBACK	AboutProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);
void Zoom(HWND hWnd, BOOL bEnable, int nLevel, int nFreq);

/******************************************************************************/
int my_atoi(const char *str)
{
	int val = 0;
	const char *p;

	p = str;
	if (str[0] == '-')
		p = str + 1;
	for (; *p; p ++)
	{
		val = val * 10 + (*p - '0');
	}
	if (str[0] == '-')
		val = -val;
	return val;
}

/******************************************************************************/
#ifdef _DEBUG
int APIENTRY WinMain(HINSTANCE hInstance,
                     HINSTANCE hPrevInstance,
                     LPSTR     lpCmdLine,
                     int       nCmdShow)
#else
void __cdecl WinMainCRTStartup(void)
#endif
{
	WNDCLASS	wndclass;
	MSG			msg;

	g_hInst = GetModuleHandle(NULL);
//  Register the window class.
	wndclass.style			= CS_HREDRAW | CS_VREDRAW;
	wndclass.lpfnWndProc	= (WNDPROC)WndProc;
	wndclass.cbClsExtra		= 0;
	wndclass.cbWndExtra		= 0;
	wndclass.hInstance		= g_hInst;
	wndclass.hIcon			= LoadIcon(g_hInst, (LPCTSTR)IDI_APPLICATION);
	wndclass.hCursor		= LoadCursor(NULL, IDC_ARROW);
	wndclass.hbrBackground	= (HBRUSH)(COLOR_WINDOW+1);
	wndclass.lpszMenuName	= (LPCSTR)IDR_MENU;
	wndclass.lpszClassName	= "deskzoom32";
	if (!RegisterClass(&wndclass))
	{
		ExitProcess(-1);
		return;
	}
//  Create & show main window
	g_hWnd = CreateWindow("deskzoom32", "DeskZoom", WS_OVERLAPPEDWINDOW,
					CW_USEDEFAULT, 0, 250, 250, NULL, NULL, g_hInst, NULL);
	if (!g_hWnd)
	{
		ExitProcess(-1);
		return;
	}
	ShowWindow(g_hWnd, SW_SHOWDEFAULT);
	UpdateWindow(g_hWnd);
// Enter message loop
	while (GetMessage(&msg, NULL, 0, 0)) 
	{
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}
	ExitProcess(msg.wParam);
}

/******************************************************************************/
LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	int wmId, wmEvent;
	PAINTSTRUCT ps;
	HDC hdc, hdcDesktop;
	RECT rt, tmp;
	int		w, h, cw, ch;
	POINT	pt;

	switch (message) 
	{
		case WM_CREATE:
			Zoom(hWnd, TRUE, 200, 50);
			break;
		case WM_COMMAND:
			wmId    = LOWORD(wParam); 
			wmEvent = HIWORD(wParam); 
			// Parse the menu selections:
			switch (wmId)
			{
				case ID_HELP_ABOUT:
				   DialogBox(g_hInst, (LPCTSTR)IDD_ABOUT, hWnd, (DLGPROC)AboutProc);
				   break;
				case ID_FILE_EXIT:
				   DestroyWindow(hWnd);
				   break;
				case ID_ZOOM_HALF:
					Zoom(hWnd, TRUE, 50, g_nFreq);
					break;
				case ID_ZOOM_NORMAL:
					Zoom(hWnd, TRUE, 100, g_nFreq);
					break;
				case ID_ZOOM_DOUBLE:
					Zoom(hWnd, TRUE, 200, g_nFreq);
					break;
				case ID_ZOOM_CUSTOM:
					g_bNoRepaint = TRUE;
					DialogBox(g_hInst, (LPCTSTR)IDD_CUSTOMIZE, hWnd, (DLGPROC)CustomizeProc);
					g_bNoRepaint = FALSE;
					Zoom(hWnd, TRUE, g_nZoomLevel, g_nFreq);
					break;
				default:
				   return DefWindowProc(hWnd, message, wParam, lParam);
			}
			break;
		case WM_PAINT:
			hdc = BeginPaint(hWnd, &ps);
			GetClientRect(hWnd, &rt);
			if (g_bNoRepaint)
			{
				FillRect(hdc, &rt, (HBRUSH)GetStockObject(WHITE_BRUSH));
			}
			else
			{
				hdcDesktop = GetDC(HWND_DESKTOP);
				GetClipBox(hdcDesktop, &tmp);
				GetCursorPos(&pt);
				w = min((int)((rt.right - rt.left) * 100 / g_nZoomLevel),
							(tmp.right - tmp.left) - pt.x);
				h = min((int)((rt.bottom - rt.top) * 100 / g_nZoomLevel),
							(tmp.bottom - tmp.top) - pt.y);
				cw = w * g_nZoomLevel / 100;
				ch = h * g_nZoomLevel / 100;
				StretchBlt(hdc, 0, 0, cw, ch, hdcDesktop, pt.x, pt.y, w, h, SRCCOPY);
				ReleaseDC(HWND_DESKTOP, hdcDesktop);
				tmp.left = cw; tmp.top = 0;
				tmp.right = rt.right - rt.left; tmp.bottom = ch;
				FillRect(hdc, &tmp, (HBRUSH)GetStockObject(WHITE_BRUSH));
				tmp.left = 0; tmp.top = ch;
				tmp.right = rt.right - rt.left;
				tmp.bottom = rt.bottom - rt.top;
				FillRect(hdc, &tmp, (HBRUSH)GetStockObject(WHITE_BRUSH));
			}
			EndPaint(hWnd, &ps);
			break;
		case WM_DESTROY:
			PostQuitMessage(0);
			break;
		case WM_TIMER:
			InvalidateRect(hWnd, NULL, FALSE);
			break;
		case WM_SETFOCUS:
			Zoom(hWnd, TRUE, g_nZoomLevel, g_nFreq);
			break;
		case WM_KILLFOCUS:
			Zoom(hWnd, FALSE, g_nZoomLevel, g_nFreq);
			break;
		default:
			return DefWindowProc(hWnd, message, wParam, lParam);
   }
   return 0;
}

/******************************************************************************/
void Zoom(HWND hWnd, BOOL bEnable, int nLevel, int nFreq)
{
	HMENU		hMenu;

	g_bNoRepaint = !bEnable;
	g_nZoomLevel = nLevel;
	g_nFreq = nFreq;
	KillTimer(hWnd, ZOOM_TIMER_ID);
	if (bEnable)
	{
		SetTimer(hWnd, ZOOM_TIMER_ID, nFreq, NULL);
	}
	hMenu = GetMenu(hWnd);
	hMenu = GetSubMenu(hMenu, 1);
	CheckMenuItem(hMenu, ID_ZOOM_HALF,
			MF_BYCOMMAND | ((nLevel == 50) ? MF_CHECKED : MF_UNCHECKED));
	CheckMenuItem(hMenu, ID_ZOOM_NORMAL,
			MF_BYCOMMAND | ((nLevel == 100) ? MF_CHECKED : MF_UNCHECKED));
	CheckMenuItem(hMenu, ID_ZOOM_DOUBLE,
		MF_BYCOMMAND | ((nLevel == 200) ? MF_CHECKED : MF_UNCHECKED));
	CheckMenuItem(hMenu, ID_ZOOM_CUSTOM,
		MF_BYCOMMAND | ((nLevel != 50 && nLevel != 100 && nLevel != 200) ? MF_CHECKED : MF_UNCHECKED));
	InvalidateRect(hWnd, NULL, FALSE);
}

/******************************************************************************/
INT_PTR CALLBACK CustomizeProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	char		szTmp[64];
	int			nFreq, nZoomLevel;

	switch (uMsg)
	{
		case WM_INITDIALOG:
				wsprintf(szTmp, "%d", g_nFreq);
				SetDlgItemText(hwndDlg, IDC_FREQ, szTmp);
				wsprintf(szTmp, "%d", g_nZoomLevel);
				SetDlgItemText(hwndDlg, IDC_LEVEL, szTmp);
				return TRUE;
		case WM_COMMAND:
			if (LOWORD(wParam) == IDOK)
			{
				GetDlgItemText(hwndDlg, IDC_FREQ, szTmp, sizeof(szTmp) - 1);
				nFreq = my_atoi(szTmp);
				if (nFreq < 10)
				{
					MessageBox(hwndDlg, "Invalid update frequency. Must be greater than 10 ms",
								"Error", MB_OK | MB_ICONERROR);
					SetFocus(GetDlgItem(hwndDlg, IDC_FREQ));
					return FALSE;
				}
				GetDlgItemText(hwndDlg, IDC_LEVEL, szTmp, sizeof(szTmp) - 1);
				nZoomLevel = my_atoi(szTmp);
				if (nZoomLevel < 2)
				{
					MessageBox(hwndDlg, "Invalid zoom level. Must be greater than 1 %",
								"Error", MB_OK | MB_ICONERROR);
					SetFocus(GetDlgItem(hwndDlg, IDC_LEVEL));
					return FALSE;
				}
				g_nFreq = nFreq;
				g_nZoomLevel = nZoomLevel;
				EndDialog(hwndDlg, LOWORD(wParam));
				return TRUE;
			}
			else if (LOWORD(wParam) == IDCANCEL) 
			{
				EndDialog(hwndDlg, LOWORD(wParam));
				return TRUE;
			}
			break;
	}
    return FALSE;
}

/******************************************************************************/
INT_PTR CALLBACK AboutProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch (uMsg)
	{
		case WM_INITDIALOG:
				return TRUE;

		case WM_COMMAND:
			if (LOWORD(wParam) == IDOK || LOWORD(wParam) == IDCANCEL) 
			{
				EndDialog(hwndDlg, LOWORD(wParam));
				return TRUE;
			}
			break;
	}
	return FALSE;
}
